package com.vendor.burn.model.devices.burners;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.commands.CrcCommand;
import com.vendor.burn.commands.I2cCommand;
import com.vendor.burn.model.BurnContext;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.FlashType;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.TransferWay;
import com.vendor.burn.model.cmdline.BurnStatus;
import com.vendor.burn.model.config.BurnConfig;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.define.IErasable;
import com.vendor.burn.model.define.IFastbootProgressMonitor;
import com.vendor.burn.model.define.IUploadable;
import com.vendor.burn.model.devices.ICommonBurner;
import com.vendor.burn.model.devices.IDeviceSetting;
import com.vendor.burn.model.devices.ITab;
import com.vendor.burn.model.devices.burners.EmmcBurner;
import com.vendor.burn.model.devices.burners.MergeBurner;
import com.vendor.burn.model.devices.unknown.UnknownDevice;
import com.vendor.burn.model.event.EventType;
import com.vendor.burn.model.event.FastbootEvent;
import com.vendor.burn.model.monitor.CmdMonitor;
import com.vendor.burn.model.receive.ReceiveTimeOutException;
import com.vendor.burn.packagers.nand.FlashOption;
import com.vendor.burn.packagers.nand.NandImageDialog;
import com.vendor.burn.packagers.nand.ip.NandIPCommon;
import com.vendor.burn.serial.workers.AbstractBurnerWorker;
import com.vendor.burn.serial.workers.CommonBurnerWorker;
import com.vendor.burn.serial.workers.EmmcPartitionBurnerWorker;
import com.vendor.burn.serial.workers.PartitionBurnerWorker;
import com.vendor.burn.sparse.SparseFileAgent;
import com.vendor.burn.ui.burners.tabs.CommonTab;
import com.vendor.burn.ui.dialog.SelectNandImageDialog;
import com.vendor.burn.utilitily.ToolUtils;
import com.vendor.burn.views.FastbootView;
import java.io.File;
import java.io.IOException;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeListener;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.vendor.tftp.tool.TFTPService;

public class CommonBurner extends EmmcBurner implements ICommonBurner, IErasable, IUploadable {
  public static boolean isSocketTransfer;
  
  private long spiTotalLen = 0L;
  
  private long nandTotalLen = 0L;
  
  private long emmcTotalLen = 0L;
  
  private String saveNandImgPath;
  
  public CommonBurner(BurnContext burnContext) {
    super(burnContext);
    getChildren().clear();
    PartitionItem fastboot = new PartitionItem();
    fastboot.setName("fastboot");
    fastboot.setFileSystem(FileSystem.none);
    getChildren().add(fastboot);
  }
  
  public int commonBurn(IFastbootProgressMonitor fastbootMonitor, CommonBurnerWorker worker) {
    boolean result = false;
    int[] errorCode = new int[1];
    errorCode[0] = BurnStatus.SUCCESS.getErrorCode();
    try {
      FastbootEvent event = fastbootMonitor.getEvent();
      if (!ToolUtils.isCmdLine()) {
        Job fastbootJob = startBurnJob(fastbootMonitor, worker);
        closeBurnJobThread(fastbootMonitor, fastbootJob);
      } else {
        if (!initFastbootMonitor(fastbootMonitor, EventType.BURN))
          return BurnStatus.NULL_POINTER_ERROR.getErrorCode(); 
        boolean initResult = initTransferway(fastbootMonitor, getBurnContext(), worker);
        if (!initResult)
          return BurnStatus.INIT_TRANSFER_WAY_FAILED.getErrorCode(); 
        CmdMonitor cmdMonitor = new CmdMonitor();
        int selectedPartCount = worker.getSelectedPartCount(this.children);
        result = worker.burnByTransferWay((IProgressMonitor)cmdMonitor, this.children, selectedPartCount, errorCode);
        event.setSuccess(result);
      } 
    } catch (IOException e) {
      errorCode[0] = BurnStatus.IOEXCEPTION_ERROR.getErrorCode();
    } catch (ReceiveTimeOutException e) {
      errorCode[0] = BurnStatus.RECEIVE_TIMEOUT_ERROR.getErrorCode();
    } catch (Throwable e) {
      errorCode[0] = BurnStatus.THROWABLE_ERROR.getErrorCode();
      fastbootMonitor.getEvent().setMessage(e.getMessage());
    } finally {
      if (!result && errorCode[0] == BurnStatus.SUCCESS.getErrorCode())
        errorCode[0] = BurnStatus.BURN_PARTITION_FAILED.getErrorCode(); 
      closeByTrans();
      if (ToolUtils.isCmdLine() && 
        fastbootMonitor != null)
        fastbootMonitor.finish(); 
    } 
    return errorCode[0];
  }
  
  public void burn(IFastbootProgressMonitor fastbootMonitor) {
    if (!initFastbootMonitor(fastbootMonitor, EventType.BURN))
      return; 
    BurnContext context = getBurnContext();
    CommonBurnerWorker worker = getBurnerWorker();
    boolean initResult = initTransferway(fastbootMonitor, context, worker);
    if (!initResult)
      return; 
    commonBurn(fastbootMonitor, worker);
  }
  
  public int erase(IFastbootProgressMonitor fastbootMonitor, CommonBurnerWorker worker) {
    if (fastbootMonitor == null)
      return BurnStatus.NULL_POINTER_ERROR.getErrorCode(); 
    boolean result = false;
    int[] errorCode = new int[1];
    try {
      FastbootEvent event = fastbootMonitor.getEvent();
      worker.setProgressMonitor(fastbootMonitor);
      CmdMonitor cmdMonitor = new CmdMonitor();
      result = worker.erase((IProgressMonitor)cmdMonitor, this.children);
      event.setSuccess(result);
    } catch (IOException iOException) {
    
    } catch (ReceiveTimeOutException receiveTimeOutException) {
    
    } catch (Throwable e) {
      fastbootMonitor.getEvent().setMessage(e.getMessage());
    } finally {
      if (!result && errorCode[0] == BurnStatus.SUCCESS.getErrorCode())
        errorCode[0] = BurnStatus.ERASE_PARTITION_FAILED.getErrorCode(); 
      closeByTrans();
      if (ToolUtils.isCmdLine() && 
        fastbootMonitor != null)
        fastbootMonitor.finish(); 
    } 
    return errorCode[0];
  }
  
  public void erase(IFastbootProgressMonitor fastbootMonitor) {
    if (!initFastbootMonitor(fastbootMonitor, EventType.ERASE) || fastbootMonitor == null)
      return; 
    fastbootMonitor.begin();
    CommonBurnerWorker worker = getBurnerWorker();
    Job fastbootJob = startEarseJob(fastbootMonitor, worker);
    closeEraseJob(fastbootMonitor, worker, fastbootJob);
  }
  
  public void eraseAll(IFastbootProgressMonitor fastbootMonitor) {
    if (!initFastbootMonitor(fastbootMonitor, EventType.ERASE) || fastbootMonitor == null)
      return; 
    fastbootMonitor.begin();
    CommonBurnerWorker worker = getBurnerWorker();
    Job fastbootJob = startEraseAllJob(fastbootMonitor, worker);
    closeEraseAllJob(fastbootMonitor, worker, fastbootJob);
  }
  
  public void upload(IFastbootProgressMonitor fastbootMonitor) {
    if (!initFastbootMonitor(fastbootMonitor, EventType.UPLOAD))
      return; 
    BurnContext context = getBurnContext();
    CommonBurnerWorker worker = getBurnerWorker();
    boolean initResult = initTransferway(fastbootMonitor, context, worker);
    if (!initResult)
      return; 
    Job job = startUploadJob(fastbootMonitor, worker);
    closeUploadThread(fastbootMonitor, job);
  }
  
  public void createImage(IFastbootProgressMonitor fastbootMonitor) {
    if (!initFastbootMonitor(fastbootMonitor, EventType.CREATE_IMAGE))
      return; 
    CommonBurnerWorker worker = getBurnerWorker();
    worker.setProgressMonitor(fastbootMonitor);
    FastbootEvent event = fastbootMonitor.getEvent();
    List<PartitionItem> parts = getChildren();
    if (FlashType.isOnlyEmmc(parts, FlashType.emmc) || 
      FlashType.isOnlySelectFlashType(parts, FlashType.ufs)) {
      makeEmmcBurnImage(fastbootMonitor, worker, event);
    } else if (FlashType.isOnlySelectFlashType(parts, FlashType.nand)) {
      createNandImage(fastbootMonitor, parts);
    } else if (FlashType.isOnlySelectFlashType(parts, FlashType.spi)) {
      MergeBurner mergeBurner = new MergeBurner(getBurnContext());
      mergeBurner.setChildren(parts);
      mergeBurner.mergeImage(fastbootMonitor);
    } 
  }
  
  public CommonBurnerWorker getBurnerWorker() {
    return new CommonBurnerWorker(this);
  }
  
  public ITab getTab() {
    return (ITab)new CommonTab();
  }
  
  public IStatus checkBeforeBurn() {
    IStatus result = checkTransferWay();
    if (!isOK(result))
      return result; 
    result = checkTable(EventType.BURN);
    if (!isOK(result))
      return result; 
    if (!ToolUtils.isCmdLine())
      if (!isOK(checkHasSelectNullFile()))
        return Status.CANCEL_STATUS;  
    return Status.OK_STATUS;
  }
  
  public IStatus checkBeforeErase() {
    IStatus check = super.checkBeforeBurn();
    if (!isOK(check))
      return check; 
    check = checkTable(EventType.ERASE);
    if (!isOK(check))
      return check; 
    return Status.OK_STATUS;
  }
  
  public IStatus checkBeforeUpload() {
    IStatus validLocalConfig = validSerialPort();
    if (!isOK(validLocalConfig))
      return validLocalConfig; 
    IStatus check = validNetConfig();
    if (!isOK(check))
      return check; 
    TransferWay transferWay = getBurnContext().getTransferWay();
    boolean isUsb = (transferWay == TransferWay.USBBootrom || transferWay == TransferWay.USBNonNude || transferWay == TransferWay.UsbAndSerial);
    if (transferWay == TransferWay.I2CBootrom)
      return error(NLS.bind(Messages.AddressComposite_Current_NotSupport_Upload, "I2C")); 
    if (isUsb)
      return error(NLS.bind(Messages.AddressComposite_Current_NotSupport_Upload, "USB")); 
    if (transferWay == TransferWay.Serial)
      return error(NLS.bind(Messages.AddressComposite_Current_NotSupport_Upload, Messages.FastbootView_TransferWay_Serial)); 
    check = checkTable(EventType.UPLOAD);
    if (!isOK(check))
      return check; 
    return null;
  }
  
  public IStatus checkCreateImage() {
    List<PartitionItem> parts = getChildren();
    if (parts.isEmpty())
      return null; 
    IStatus result = checkTable(EventType.BURN);
    if (!isOK(result))
      return result; 
    for (int i = 0; i < parts.size(); i++) {
      PartitionItem partition = this.children.get(i);
      if (partition.isChecked()) {
        FlashType flashType = partition.getFlashType();
        File file = partition.getFile();
        if (flashType == FlashType.ssd) {
          highLight(partition);
          return error(NLS.bind(Messages.PartitionBurner_msg_invalid_flash_type_ssd, Integer.valueOf(i + 1)));
        } 
        if (flashType == FlashType.nand) {
          if (file == null) {
            highLight(partition);
            return error(NLS.bind(Messages.AbstractBurner_msg_file_path_does_not_select_in_row_, Integer.valueOf(i + 1)));
          } 
          if (!file.exists()) {
            highLight(partition);
            return error(NLS.bind(Messages.AbstractBurner_msg_file_does_not_exist_in_row_, Integer.valueOf(i + 1)));
          } 
          if (file.isDirectory()) {
            highLight(partition);
            return error(NLS.bind(Messages.AbstractBurner_msg_the_file_can_not_be_a_directory, Integer.valueOf(i + 1)));
          } 
        } 
      } 
    } 
    ArrayList<FlashType> hasFlashTypes = new ArrayList<>();
    for (PartitionItem partitionItem : parts) {
      FlashType flashType = partitionItem.getFlashType();
      if (!hasFlashTypes.contains(flashType))
        hasFlashTypes.add(flashType); 
    } 
    if (hasFlashTypes.contains(FlashType.emmc) && hasFlashTypes.contains(FlashType.nand))
      return error(Messages.CommonComposite_Check_Burn_Image_NotSupport_Mixed); 
    return Status.OK_STATUS;
  }
  
  public IStatus checkCreateProImage() {
    List<PartitionItem> parts = getChildren();
    if (parts.isEmpty())
      return null; 
    IStatus result = checkTable(EventType.BURN);
    if (!isOK(result))
      return result; 
    for (int i = 0; i < parts.size(); i++) {
      PartitionItem partition = this.children.get(i);
      if (partition.isChecked()) {
        FlashType flashType = partition.getFlashType();
        File file = partition.getFile();
        if (flashType == FlashType.nand) {
          if (file == null) {
            highLight(partition);
            return error(NLS.bind(Messages.AbstractBurner_msg_file_path_does_not_select_in_row_, Integer.valueOf(i + 1)));
          } 
          if (!file.exists()) {
            highLight(partition);
            return error(NLS.bind(Messages.AbstractBurner_msg_file_does_not_exist_in_row_, Integer.valueOf(i + 1)));
          } 
          if (file.isDirectory()) {
            highLight(partition);
            return error(NLS.bind(Messages.AbstractBurner_msg_the_file_can_not_be_a_directory, Integer.valueOf(i + 1)));
          } 
        } 
      } 
    } 
    ArrayList<FlashType> hasFlashTypes = new ArrayList<>();
    for (PartitionItem partitionItem : parts) {
      FlashType flashType = partitionItem.getFlashType();
      if (!hasFlashTypes.contains(flashType))
        hasFlashTypes.add(flashType); 
    } 
    if (hasFlashTypes.contains(FlashType.emmc) && (hasFlashTypes
      .contains(FlashType.nand) || hasFlashTypes.contains(FlashType.spi)))
      return error(Messages.CommonComposite_Check_Burn_Image_NotSupport_Mixed); 
    return Status.OK_STATUS;
  }
  
  protected IStatus checkTable(EventType event) {
    List<PartitionItem> parts = getChildren();
    IStatus check = checkPartSelect(parts);
    if (!isOK(check))
      return check; 
    check = checkFastbootPart(parts.get(0), event);
    if (!isOK(check)) {
      if (!ToolUtils.isCmdLine())
        getBurnContext().getUIAdapter().highLight(parts.get(0)); 
      return check;
    } 
    check = checkOtherPart(parts, event);
    if (!isOK(check))
      return check; 
    return Status.OK_STATUS;
  }
  
  protected IStatus checkOtherPart(List<PartitionItem> parts, EventType event) {
    FlashType[] flashTypes = FlashType.getFlashTypes(this);
    FileSystem[] fileSystems = FileSystem.getFileSystems(this);
    long spiLastAddress = 0L;
    long nandLastAddress = 0L;
    long emmcLastAddress = 0L;
    IStatus result = Status.OK_STATUS;
    PartitionItem fastboot = parts.get(0);
    this.spiTotalLen = 0L;
    this.nandTotalLen = 0L;
    this.emmcTotalLen = 0L;
    long fastbootPartLen = fastboot.getLength();
    if (FlashType.spi == fastboot.getFlashType()) {
      this.spiTotalLen = fastbootPartLen;
    } else if (FlashType.nand == fastboot.getFlashType()) {
      this.nandTotalLen = fastbootPartLen;
    } else if (FlashType.emmc == fastboot.getFlashType()) {
      this.emmcTotalLen = fastbootPartLen;
    } 
    for (int partIndex = 1; partIndex < parts.size(); partIndex++) {
      PartitionItem part = parts.get(partIndex);
      if (!part.isChecked())
        continue; 
      result = checkFlashAndFileSystem(partIndex, flashTypes, fileSystems, part);
      if (result != Status.OK_STATUS)
        return result; 
      result = checkStartAddressAndLen(part, parts.size(), partIndex);
      if (result != Status.OK_STATUS)
        return result; 
      result = checkAddressOverlap(partIndex, part);
      if (result != Status.OK_STATUS)
        return result; 
      long start = part.getStart();
      if (0L == part.getLength() && part.getFlashType() != FlashType.ssd) {
        highLight(part);
        return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_length_must_be_nonzero_in_row_, 
              Integer.valueOf(partIndex + 1)));
      } 
      boolean isCheck = !FastbootView.isChipBranchSeries8();
      if (FlashType.spi == part.getFlashType() && isCheck) {
        if (start < spiLastAddress) {
          highLight(part);
          return error(NLS.bind(Messages.AbstractBurner_messgae_the_partition_address_must_be_greater_than_the_used_size_in_row_, 
                
                Integer.valueOf(partIndex + 1)));
        } 
        spiLastAddress = start + ((part.getLength() == -1L) ? 0L : part.getLength());
      } else if (FlashType.nand == part.getFlashType() && isCheck) {
        if (start < nandLastAddress) {
          highLight(part);
          return error(NLS.bind(Messages.AbstractBurner_messgae_the_partition_address_must_be_greater_than_the_used_size_in_row_, 
                
                Integer.valueOf(partIndex + 1)));
        } 
        nandLastAddress = start + ((part.getLength() == -1L) ? 0L : part.getLength());
      } else if (FlashType.emmc == part.getFlashType() && isCheck) {
        if (start < emmcLastAddress) {
          highLight(part);
          return error(NLS.bind(Messages.AbstractBurner_messgae_the_partition_address_must_be_greater_than_the_used_size_in_row_, 
                
                Integer.valueOf(partIndex + 1)));
        } 
        emmcLastAddress = start + ((part.getLength() == -1L) ? 0L : part.getLength());
      } 
      if (!part.isChecked())
        continue; 
      File file = part.getFile();
      if (EventType.BURN == event) {
        result = checkBurn(parts.size(), partIndex, part, file);
      } else {
        if (EventType.UPLOAD == event) {
          result = checkUpload(parts.size(), partIndex, part, file);
          if (result != Status.OK_STATUS)
            return result; 
          continue;
        } 
        if (EventType.ERASE == event)
          continue; 
      } 
      if (result != Status.OK_STATUS)
        return result; 
      result = checkStartAndLen(part, partIndex, parts.size());
      if (result != Status.OK_STATUS)
        return result; 
      result = checkNetFileNameLen(partIndex, file);
      if (result != Status.OK_STATUS)
        return result; 
      continue;
    } 
    return Status.OK_STATUS;
  }
  
  private boolean initFastbootMonitor(IFastbootProgressMonitor fastbootMonitor, EventType evenType) {
    if (fastbootMonitor == null)
      return false; 
    fastbootMonitor.getEvent().init(evenType);
    this.worker.setProgressMonitor(fastbootMonitor);
    return true;
  }
  
  private Job startBurnJob(IFastbootProgressMonitor fastbootMonitor, CommonBurnerWorker worker) {
    FastbootEvent event = fastbootMonitor.getEvent();
    Object object = new Object(this, "Burn By Partition", worker, event, fastbootMonitor);
    showProgressDialog((Job)object);
    return (Job)object;
  }
  
  private void closeBurnJobThread(IFastbootProgressMonitor fastbootMonitor, Job fastbootJob) {
    fastbootJob.addJobChangeListener((IJobChangeListener)new Object(this, fastbootMonitor));
    fastbootJob.schedule();
  }
  
  private Job startEarseJob(IFastbootProgressMonitor fastbootMonitor, CommonBurnerWorker worker) {
    return (Job)new Object(this, "Erase By Partition", worker, fastbootMonitor);
  }
  
  private void closeEraseJob(IFastbootProgressMonitor fastbootMonitor, CommonBurnerWorker worker, Job fastbootJob) {
    showProgressDialog(fastbootJob);
    fastbootJob.addJobChangeListener((IJobChangeListener)new Object(this, fastbootMonitor));
    fastbootJob.schedule();
  }
  
  private Job startUploadJob(IFastbootProgressMonitor fastbootMonitor, CommonBurnerWorker worker) {
    FastbootEvent event = fastbootMonitor.getEvent();
    event.setBurner((IBurner)this);
    return (Job)new Object(this, "Upload By Partition", worker, event, fastbootMonitor);
  }
  
  private void closeUploadThread(IFastbootProgressMonitor fastbootMonitor, Job job) {
    showProgressDialog(job);
    job.addJobChangeListener((IJobChangeListener)new Object(this, fastbootMonitor));
    job.schedule();
  }
  
  private boolean initTransferway(IFastbootProgressMonitor fastbootMonitor, BurnContext context, CommonBurnerWorker worker) {
    if (!initUart(fastbootMonitor, context, worker))
      return false; 
    boolean isRemoteTftpServer = context.getLocalConfig().isRemoteTftpServer();
    context.setTftpOccupied(isRemoteTftpServer);
    if (!isRemoteTftpServer && 
      !initNetWork(fastbootMonitor, context, worker))
      return false; 
    if (!initI2CBootrom(fastbootMonitor, context, worker))
      return false; 
    return true;
  }
  
  private boolean initI2CBootrom(IFastbootProgressMonitor fastbootMonitor, BurnContext context, CommonBurnerWorker worker) {
    if (context.getTransferWay() == TransferWay.I2CBootrom)
      if (I2cCommand.I2cOpenChannelForBootrom(52, fastbootMonitor)) {
        CommonBurnerWorker.infoln("\nI2C channels open success, the bit rate is " + I2cCommand.getBaudRate() + ", device address is 52");
      } else {
        CommonBurnerWorker.infoln("I2C channels open failed");
        return false;
      }  
    return true;
  }
  
  private boolean initUart(IFastbootProgressMonitor fastbootMonitor, BurnContext context, CommonBurnerWorker worker) {
    if (fastbootMonitor != null)
      fastbootMonitor.begin(); 
    return true;
  }
  
  private boolean initNetWork(IFastbootProgressMonitor fastbootMonitor, BurnContext context, CommonBurnerWorker worker) {
    if (worker.isNeedSetupNet(getCurTransferWay())) {
      boolean start = false;
      boolean portCheck = false;
      int port = CrcCommand.getTFTPPort();
      try {
        portCheck = TFTPService.TFTPSERVICE.checkPort(ToolUtils.getPortCmd(), port);
        if (portCheck) {
          start = TFTPService.TFTPSERVICE.start(getParentFile(), null, port);
        } else {
          fastbootMonitor.warnln(Messages.PartitionBurner_msg_start_Tftp_server_on_port_69_is_used);
        } 
        getTftpPreferenceValue();
        context.setTftpOccupied(!start);
      } catch (SocketException e) {
        context.setTftpOccupied(true);
        handleTftpStartFailed();
      } finally {
        if (!start && portCheck) {
          fastbootMonitor.warnln(Messages.PartitionBurner_msg_start_Tftp_server_start);
          return false;
        } 
      } 
    } 
    return true;
  }
  
  private File getParentFile() {
    if (this.children == null || this.children.isEmpty())
      return null; 
    for (PartitionItem partition : this.children) {
      if (partition.isChecked() && 
        partition.getFile() != null)
        return partition.getFile().getParentFile(); 
    } 
    return null;
  }
  
  private IStatus checkPartSelect(List<PartitionItem> parts) {
    int selection = 0;
    for (PartitionItem partitionItem : parts) {
      if (partitionItem.isChecked()) {
        selection++;
        break;
      } 
    } 
    String ethaddrStr = BurnConfig.queryForString("ethaddr");
    if (0 == selection && (ethaddrStr == null || ethaddrStr.isEmpty()))
      return error(Messages.AbstractBurner_msg_emmc_no_createTable_and_non_partition); 
    return Status.OK_STATUS;
  }
  
  private IStatus checkTransferWay() {
    TransferWay transferWay = getCurTransferWay();
    boolean needCheckSerial = (transferWay != TransferWay.Usb2I2c && transferWay != TransferWay.I2CBootrom && transferWay != TransferWay.USBBootrom && transferWay != TransferWay.USBNonNude && transferWay != TransferWay.JTAGAndNet && transferWay != TransferWay.JTAG && transferWay != TransferWay.JTAGAndSerial);
    if (needCheckSerial) {
      IStatus check = validSerialPort();
      if (!isOK(check))
        return check; 
    } 
    boolean needCheckTftp = (transferWay == TransferWay.Net || transferWay == TransferWay.UsbTransNet || transferWay == TransferWay.JTAGAndNet);
    if (needCheckTftp) {
      IStatus check = validNetConfig();
      if (!isOK(check))
        return check; 
    } 
    List<PartitionItem> parts = getChildren();
    boolean hasEmmcFlash = false;
    ArrayList<FlashType> hasFlashTypes = FlashType.getCurFlashTypes(parts);
    for (FlashType flashType : hasFlashTypes) {
      if (flashType == FlashType.emmc) {
        hasEmmcFlash = true;
        break;
      } 
    } 
    boolean supportUart2Compress = (!hasEmmcFlash && transferWay == TransferWay.Uart2Compress);
    if (supportUart2Compress)
      return error(NLS.bind(Messages.CommonComposite_Check_EmmcFlash_NotSupport_Current_Transmission, FlashType.emmc
            .name(), Messages.FastbootView_TransferWay_Uart2Compress)); 
    boolean supportI2C = (hasEmmcFlash && (transferWay == TransferWay.I2CBootrom || transferWay == TransferWay.Usb2I2c));
    if (supportI2C)
      return error(NLS.bind(Messages.CommonComposite_Check_EmmcFlash_NotSupport_Current_Transmission, FlashType.emmc
            .name(), Messages.FastbootView_TransferWay_I2cBootrom)); 
    return Status.OK_STATUS;
  }
  
  private IStatus checkFlashAndFileSystem(int partIndex, FlashType[] flashTypes, FileSystem[] fileSystems, PartitionItem part) {
    boolean valid = false;
    for (FileSystem fileSystem1 : fileSystems) {
      if (fileSystem1 == part.getFileSystem()) {
        valid = true;
        break;
      } 
    } 
    if (!valid) {
      highLight(part);
      return error(NLS.bind(Messages.AbstractBurner_msg_invalid_file_system_in_row_, Integer.valueOf(partIndex + 1)));
    } 
    valid = false;
    for (FlashType flashType1 : flashTypes) {
      if (flashType1 == part.getFlashType()) {
        valid = true;
        break;
      } 
    } 
    if (FastbootView.isChipBranchSeries8())
      valid = true; 
    if (!valid) {
      highLight(part);
      return error(NLS.bind(Messages.AbstractBurner_msg_invalid_flash_type_in_row_, Integer.valueOf(partIndex + 1)));
    } 
    FlashType flashType = part.getFlashType();
    FileSystem fileSystem = part.getFileSystem();
    boolean invaildFlashForYaffs = (fileSystem == FileSystem.yaffs && flashType != FlashType.nand);
    if (invaildFlashForYaffs) {
      highLight(part);
      return error(NLS.bind(Messages.PartitionBurner_msg_flash_SPI_and_file_system_Yaffs_can_not_be_selected_at_the_same_time_in_row_, 
            Integer.valueOf(partIndex + 1)));
    } 
    boolean invaildFlashForUbifs = (fileSystem == FileSystem.ubiFs && flashType != FlashType.nand);
    if (invaildFlashForUbifs) {
      highLight(part);
      return error(NLS.bind(Messages.PartitionBurner_UBI_only_support_Nand_flash, Integer.valueOf(partIndex + 1)));
    } 
    boolean invaildFlashForExt3 = (SparseFileAgent.isSparse(fileSystem) && flashType != FlashType.emmc && flashType != FlashType.ufs && flashType != FlashType.ssd);
    if (invaildFlashForExt3) {
      highLight(part);
      return error(NLS.bind(Messages.PartitionBurner_EXT3_only_support_Emmc_flash, Integer.valueOf(partIndex + 1)));
    } 
    IStatus result = checkYaffsFile(partIndex, part, fileSystem);
    if (result != Status.OK_STATUS)
      return result; 
    return Status.OK_STATUS;
  }
  
  private IStatus checkAddressOverlap(int index, PartitionItem part) {
    long start = part.getStart();
    long length = part.getLength();
    FlashType flashType = part.getFlashType();
    IStatus result = Status.OK_STATUS;
    if (FlashType.spi == flashType) {
      result = checkFlashTotalLen(index, part, this.spiTotalLen, start);
      this.spiTotalLen = start + length;
    } else if (FlashType.nand == flashType) {
      result = checkFlashTotalLen(index, part, this.nandTotalLen, start);
      this.nandTotalLen = start + length;
    } else if (FlashType.emmc == flashType) {
      result = checkFlashTotalLen(index, part, this.emmcTotalLen, start);
      this.emmcTotalLen = start + length;
    } 
    if (!isOK(result))
      return result; 
    return Status.OK_STATUS;
  }
  
  private IStatus checkFlashTotalLen(int index, PartitionItem part, long totalLength, long start) {
    if (FastbootView.isChipBranchSeries8())
      return Status.OK_STATUS; 
    if (index == 1 && FastbootView.isChipBranchSeries9())
      return Status.OK_STATUS; 
    if (0L == start && 0L != totalLength) {
      highLight(part);
      return error(NLS.bind(Messages.AbstractBurner_msg_the_partition_address_must_be_nonzero_in_row_, 
            Integer.valueOf(index + 1)));
    } 
    if (0L != totalLength && start < totalLength) {
      highLight(part);
      return error(NLS.bind(Messages.PartitionBurner_msg_address_overlap_in_row_, Integer.valueOf(index + 1)));
    } 
    return Status.OK_STATUS;
  }
  
  private void closeEraseAllJob(IFastbootProgressMonitor fastbootMonitor, CommonBurnerWorker worker, Job fastbootJob) {
    showProgressDialog(fastbootJob);
    fastbootJob.addJobChangeListener((IJobChangeListener)new Object(this, fastbootMonitor));
    fastbootJob.schedule();
  }
  
  private Job startEraseAllJob(IFastbootProgressMonitor fastbootMonitor, CommonBurnerWorker worker) {
    return (Job)new Object(this, "Erase By Partition", worker, fastbootMonitor);
  }
  
  private TransferWay getCurTransferWay() {
    return getBurnContext().getTransferWay();
  }
  
  private NandIPCommon chooseNandSet(UnknownDevice device, int nandIPVer, int spinandIPVer) {
    NandIPCommon nandSet = null;
    if (nandIPVer != -1 && spinandIPVer != -1) {
      SelectNandImageDialog selectNandImageDialog = new SelectNandImageDialog(Display.getCurrent().getActiveShell());
      int open = selectNandImageDialog.open();
      if (open == 0) {
        String selectedNandWay = selectNandImageDialog.getSelectNandImageWay();
        if (selectedNandWay.equals("Nand")) {
          nandSet = NandIPCommon.chooseNandVersion(device);
        } else if (selectedNandWay.equals("SPINand")) {
          nandSet = NandIPCommon.chooseSPINandVersion(device);
        } 
        if (nandSet != null)
          nandSet.setCurSelectedNandWay(selectedNandWay); 
      } else {
        return null;
      } 
    } else if (spinandIPVer != -1) {
      nandSet = NandIPCommon.chooseSPINandVersion(device);
      if (nandSet != null)
        nandSet.setCurSelectedNandWay("SPINand"); 
    } else {
      nandSet = NandIPCommon.chooseNandVersion(device);
      if (nandSet != null)
        nandSet.setCurSelectedNandWay("Nand"); 
    } 
    return nandSet;
  }
  
  private boolean checkCurChipSupport(int nandIPVer, int spinandIPVer) {
    boolean supportNand = (nandIPVer != 500 && nandIPVer != 600 && nandIPVer != 610 && nandIPVer != 611 && nandIPVer != 620);
    boolean supportSpi = (spinandIPVer != 500 && spinandIPVer != 600 && spinandIPVer != 610 && spinandIPVer != 611 && spinandIPVer != 620);
    if (supportNand && supportSpi) {
      MessageDialog.openInformation(Display.getCurrent().getActiveShell(), "BurnTool", Messages.NandImage_DoNotSupport);
      return false;
    } 
    return true;
  }
  
  private void createNandImage(IFastbootProgressMonitor fastbootMonitor, List<PartitionItem> parts) {
    IDeviceSetting deviceSetting = getBurnContext().getDeviceSetting();
    if (deviceSetting == null) {
      MessageDialog.openInformation(Display.getCurrent().getActiveShell(), "BurnTool", Messages.PartitionComposite_MakeNandImage_DoNotSupport);
      fastbootMonitor.finish();
      return;
    } 
    UnknownDevice device = (UnknownDevice)deviceSetting;
    int nandIPVer = device.getNandIPVer();
    int spinandIPVer = device.getSpinandIPVer();
    if (!checkCurChipSupport(nandIPVer, spinandIPVer)) {
      fastbootMonitor.finish();
      return;
    } 
    NandIPCommon nandSet = chooseNandSet(device, nandIPVer, spinandIPVer);
    if (nandSet == null) {
      fastbootMonitor.finish();
      return;
    } 
    NandImageDialog dialog = new NandImageDialog(Display.getCurrent().getActiveShell(), 67680, nandSet, (IBurner)this);
    dialog.open();
    FlashOption option = dialog.getSelectedFlashOption();
    NandIPCommon nandSetting = nandSet;
    int blockSize = dialog.getBlockSize();
    if (option == null) {
      fastbootMonitor.finish();
      return;
    } 
    boolean isSelectRandom = dialog.isSelectRandom();
    DirectoryDialog outputSelectDialog = new DirectoryDialog(Display.getCurrent().getActiveShell());
    outputSelectDialog.setText(Messages.NandImageDialog_DstFolder);
    outputSelectDialog.setMessage(Messages.NandImageDialog_DstFolderSelect);
    if (this.saveNandImgPath != null)
      outputSelectDialog.setFilterPath(this.saveNandImgPath); 
    String path = outputSelectDialog.open();
    if (path == null) {
      fastbootMonitor.finish();
      return;
    } 
    this.saveNandImgPath = path;
    Object object = new Object(this, "Create Nand Burner Image...", nandSetting, path, option, parts, blockSize, isSelectRandom, fastbootMonitor);
    showProgressDialog((Job)object);
    object.addJobChangeListener((IJobChangeListener)new Object(this));
    object.schedule();
  }
  
  private void makeEmmcBurnImage(IFastbootProgressMonitor fastbootMonitor, CommonBurnerWorker worker, FastbootEvent event) {
    FileDialog dialog = new FileDialog(Display.getDefault().getActiveShell(), 8192);
    String open = dialog.open();
    if (open == null) {
      fastbootMonitor.finish();
      return;
    } 
    File file = new File(open);
    if (file.isDirectory()) {
      MessageDialog.openWarning(Display.getDefault().getActiveShell(), Messages.FASTBOOT_WARNING_Title, "Can not be a Directory!");
      fastbootMonitor.finish();
    } 
    Object object = new Object(this, "Making Image...", worker, file, event, fastbootMonitor);
    showProgressDialog((Job)object);
    object.addJobChangeListener((IJobChangeListener)new Object(this, fastbootMonitor));
    object.schedule();
  }
  
  public int burnZip(IFastbootProgressMonitor fastbootMonitor, CommonBurnerWorker worker) {
    boolean result = false;
    int[] errorCode = new int[1];
    errorCode[0] = BurnStatus.SUCCESS.getErrorCode();
    try {
      FastbootEvent event = fastbootMonitor.getEvent();
      if (!ToolUtils.isCmdLine()) {
        Job fastbootJob = startBurnJob(fastbootMonitor, worker);
        closeBurnJobThread(fastbootMonitor, fastbootJob);
      } else {
        if (!initFastbootMonitor(fastbootMonitor, EventType.BURN))
          return BurnStatus.NULL_POINTER_ERROR.getErrorCode(); 
        boolean initResult = initTransferway(fastbootMonitor, getBurnContext(), worker);
        if (!initResult)
          return BurnStatus.INIT_TRANSFER_WAY_FAILED.getErrorCode(); 
        CmdMonitor cmdMonitor = new CmdMonitor();
        int selectedPartCount = worker.getSelectedPartCount(this.children);
        result = worker.burnZipByTransferWay((IProgressMonitor)cmdMonitor, this.children, selectedPartCount, errorCode);
        event.setSuccess(result);
      } 
    } catch (IOException e) {
      errorCode[0] = BurnStatus.IOEXCEPTION_ERROR.getErrorCode();
    } catch (ReceiveTimeOutException e) {
      errorCode[0] = BurnStatus.RECEIVE_TIMEOUT_ERROR.getErrorCode();
    } catch (Throwable e) {
      errorCode[0] = BurnStatus.THROWABLE_ERROR.getErrorCode();
      fastbootMonitor.getEvent().setMessage(e.getMessage());
    } finally {
      if (!result && errorCode[0] == BurnStatus.SUCCESS.getErrorCode())
        errorCode[0] = BurnStatus.BURN_PARTITION_FAILED.getErrorCode(); 
      closeByTrans();
      if (ToolUtils.isCmdLine() && 
        fastbootMonitor != null)
        fastbootMonitor.finish(); 
    } 
    return errorCode[0];
  }
}
